#include "HMC5843_compass.h"
#include "imu_int.h"
#include "ram.h"

// compass registers
#define COMPASS_CONF_A         0x00
#define COMPASS_CONF_B         0x01
#define COMPASS_MODE           0x02
#define COMPASS_STATUS         0x09
#define COMPASS_IDA            0x0A
#define COMPASS_IDB            0x0B
#define COMPASS_IDC            0x0C

// configuration values
#define COMPASS_DATA_RATE_0_5  0x00
#define COMPASS_DATA_RATE_1    0x04
#define COMPASS_DATA_RATE_2    0x08
#define COMPASS_DATA_RATE_5    0x0C
#define COMPASS_DATA_RATE_10   0x10
#define COMPASS_DATA_RATE_20   0x14
#define COMPASS_DATA_RATE_50   0x18

#define COMPASS_NO_BIAS        0x00
#define COMPASS_POS_BIAS       0x01
#define COMPASS_NEG_BIAS       0x02

#define COMPASS_FS_0_7         0x00
#define COMPASS_FS_1           0x20
#define COMPASS_FS_1_5         0x40
#define COMPASS_FS_2           0x60
#define COMPASS_FS_3_2         0x80
#define COMPASS_FS_3_8         0xA0
#define COMPASS_FS_4_5         0xC0
#define COMPASS_FS_6_5         0xE0

#define COMPASS_CONT_MODE      0x00
#define COMPASS_SINGLE_MODE    0x01
#define COMPASS_IDLE_MODE      0x02
#define COMPASS_SLEEP_MODE     0x03

// compass data adn configuration
const uint8_t compass_id_reg=0x0A;
const uint8_t compass_id[3]={'H','4','3'};
const uint8_t compass_id_len=0x03;
const uint8_t compass_conf_reg=0x00;
const uint8_t compass_conf_len=0x03;
const uint8_t compass_data_reg=0x03;
const uint8_t compass_data_len=0x06;
const uint8_t compass_status_reg=0x09;
const uint8_t compass_status_len=0x01;
const uint8_t compass_rd_addr=0x3D;
const uint8_t compass_wr_addr=0x3C;
uint8_t i2c_compass_conf[3];
uint8_t i2c_compass_data[6];

// compass public functions
uint8_t imu_compass_detect(void)
{
  i2c_op=I2C_RD_OP_CMD;
  IMU_DMA_RX_InitStructure.DMA_BufferSize=compass_id_len;
  IMU_DMA_RX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_compass_data;
  DMA_Init(I2C_IMU_DMA_STREAM_RX,&IMU_DMA_RX_InitStructure);
  IMU_DMA_TX_InitStructure.DMA_BufferSize=1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)&compass_id_reg;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_compass_data[0]=0x00;
  i2c_compass_data[1]=0x00;
  i2c_compass_data[2]=0x00;
  i2c_rd_addr=compass_rd_addr;
  i2c_wr_addr=compass_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);

  imu_wait_op_done();
  if(i2c_compass_data[0]==compass_id[0] && i2c_compass_data[1]==compass_id[1] && i2c_compass_data[2]==compass_id[2])
  {
    imu_compass_get_config();
    imu_compass_set_config();
    return 0x01;
  }
  else
    return 0x00;
}

void imu_compass_get_config(void)
{
  i2c_op=I2C_RD_OP_CMD;
  IMU_DMA_RX_InitStructure.DMA_BufferSize=compass_conf_len;
  IMU_DMA_RX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)i2c_compass_conf;
  DMA_Init(I2C_IMU_DMA_STREAM_RX,&IMU_DMA_RX_InitStructure);
  IMU_DMA_TX_InitStructure.DMA_BufferSize=1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)&compass_conf_reg;
  i2c_rd_addr=compass_rd_addr;
  i2c_wr_addr=compass_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
}

void imu_compass_start(void)
{
  // enter measure mode
  i2c_compass_conf[COMPASS_MODE-COMPASS_CONF_A]=COMPASS_CONT_MODE;
  i2c_data_out[0]=COMPASS_MODE;
  i2c_data_out[1]=i2c_compass_conf[COMPASS_MODE-COMPASS_CONF_A];
  // write the fifo configuration
  i2c_op=I2C_WR_OP;
  IMU_DMA_TX_InitStructure.DMA_BufferSize=2;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_data_out;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=compass_rd_addr;
  i2c_wr_addr=compass_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
}

void imu_compass_stop(void)
{
  // enter measure mode
  i2c_compass_conf[COMPASS_MODE-COMPASS_CONF_A]=COMPASS_SLEEP_MODE;
  i2c_data_out[0]=COMPASS_MODE;
  i2c_data_out[1]=i2c_compass_conf[COMPASS_MODE-COMPASS_CONF_A];
  // write the fifo configuration
  i2c_op=I2C_WR_OP;
  IMU_DMA_TX_InitStructure.DMA_BufferSize=2;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_data_out;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=compass_rd_addr;
  i2c_wr_addr=compass_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
}

void imu_compass_get_data(void)
{
  i2c_op=I2C_RD_OP_CMD;
  IMU_DMA_RX_InitStructure.DMA_BufferSize=compass_data_len;
  IMU_DMA_RX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)i2c_compass_data;
  DMA_Init(I2C_IMU_DMA_STREAM_RX,&IMU_DMA_RX_InitStructure);
  IMU_DMA_TX_InitStructure.DMA_BufferSize=1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr=(uint32_t)&compass_data_reg;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=compass_rd_addr;
  i2c_wr_addr=compass_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
}

void imu_compass_set_config(void)
{
  uint8_t i;

  // set the configuration
  i2c_compass_conf[COMPASS_CONF_A-COMPASS_CONF_A]=COMPASS_DATA_RATE_50 | COMPASS_NO_BIAS;
  i2c_compass_conf[COMPASS_CONF_B-COMPASS_CONF_A]=COMPASS_FS_1;
  i2c_compass_conf[COMPASS_MODE-COMPASS_CONF_A]=COMPASS_SLEEP_MODE;
  // write the configuration
  for(i=0;i<compass_conf_len;i++)
    i2c_data_out[i+1]=i2c_compass_conf[i];
  i2c_data_out[0]=compass_conf_reg;
  i2c_op=I2C_WR_OP;
  IMU_DMA_TX_InitStructure.DMA_BufferSize=compass_conf_len+1;
  IMU_DMA_TX_InitStructure.DMA_Memory0BaseAddr = (uint32_t)i2c_data_out;
  DMA_Init(I2C_IMU_DMA_STREAM_TX,&IMU_DMA_TX_InitStructure);
  i2c_rd_addr=compass_rd_addr;
  i2c_wr_addr=compass_wr_addr;
  I2C_GenerateSTART(I2C_IMU, ENABLE);
  imu_wait_op_done();
}

void imu_compass_process_data(void)
{
  uint8_t i;
  int16_t value;

  for(i=0;i<3;i++)
  {
    value=(((int16_t)(i2c_compass_data[i*2+1]+(i2c_compass_data[i*2]<<8)))<<13)/1300;
    ram_data[BIOLOID_IMU_COMPASS_X_L+i*2]=value&0xFF;
    ram_data[BIOLOID_IMU_COMPASS_X_H+i*2]=(value>>8);
  }
}


