#include "bioloid_time.h"

TIM_HandleTypeDef us_timer_Handle;
unsigned long long int timer_counts;
unsigned short int timer_counts_per_us;

// interrupt service routines
void SysTick_Handler(void)
{
  HAL_IncTick();
  HAL_SYSTICK_IRQHandler();
}

void TIM6_DAC_IRQHandler(void)
{
  /* TIM Update event */
  if(__HAL_TIM_GET_FLAG(&us_timer_Handle, TIM_FLAG_UPDATE) != RESET)
  {
    if(__HAL_TIM_GET_IT_SOURCE(&us_timer_Handle, TIM_IT_UPDATE) !=RESET)
    {
      __HAL_TIM_CLEAR_IT(&us_timer_Handle, TIM_IT_UPDATE);
      timer_counts++;
    }
  }
}

// public functions
void bioloid_time_init(void)
{
  TIM_MasterConfigTypeDef sMasterConfig;

  /* initialize internal variables */
  timer_counts=0;
  timer_counts_per_us=(SystemCoreClock/2000000)+1;

  /* configure timer */
  __TIM6_CLK_ENABLE();
  us_timer_Handle.Instance = TIM6;
  us_timer_Handle.Init.Prescaler = 0;
  us_timer_Handle.Init.CounterMode = TIM_COUNTERMODE_UP;
  us_timer_Handle.Init.Period = 0xFFFF;
  HAL_TIM_Base_Init(&us_timer_Handle);

  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  HAL_TIMEx_MasterConfigSynchronization(&us_timer_Handle, &sMasterConfig);

  /* configure timer interrupts */
  HAL_NVIC_SetPriority(TIM6_DAC_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(TIM6_DAC_IRQn);
  HAL_TIM_Base_Start_IT(&us_timer_Handle);
}

unsigned long long int bioloid_time_get_counts(void)
{
  return (timer_counts<<16)+__HAL_TIM_GetCounter(&us_timer_Handle);
}

inline unsigned int bioloid_time_get_counts_per_us(void)
{
  return timer_counts_per_us;
}
