#include "gpio.h"
#include "ram.h"
#include "bioloid_dyn_slave.h"

#define ENABLE_LED_TX_GPIO_CLK      __HAL_RCC_GPIOE_CLK_ENABLE()
#define LED_TX_PIN                  GPIO_PIN_10
#define LED_TX_GPIO_PORT            GPIOE

#define ENABLE_LED_RX_GPIO_CLK      __HAL_RCC_GPIOE_CLK_ENABLE()
#define LED_RX_PIN                  GPIO_PIN_2
#define LED_RX_GPIO_PORT            GPIOE

#define ENABLE_LED_USR1_GPIO_CLK    __HAL_RCC_GPIOA_CLK_ENABLE()
#define LED_USR1_PIN                GPIO_PIN_13
#define LED_USR1_GPIO_PORT          GPIOA

#define ENABLE_LED_USR2_GPIO_CLK    __HAL_RCC_GPIOE_CLK_ENABLE()
#define LED_USR2_PIN                GPIO_PIN_9                
#define LED_USR2_GPIO_PORT          GPIOE

#define ENABLE_START_PB_GPIO_CLK    __HAL_RCC_GPIOE_CLK_ENABLE()
#define START_PB_PIN                GPIO_PIN_3
#define START_PB_GPIO_PORT          GPIOE
                       
#define ENABLE_MODE_PB_GPIO_CLK     __HAL_RCC_GPIOE_CLK_ENABLE()
#define MODE_PB_PIN                 GPIO_PIN_4
#define MODE_PB_GPIO_PORT           GPIOE
                       
#define ENABLE_RESET_PB_GPIO_CLK    __HAL_RCC_GPIOA_CLK_ENABLE()
#define RESET_PB_PIN                GPIO_PIN_12
#define RESET_PB_GPIO_PORT          GPIOA
                       
#define ENABLE_USER_PB_GPIO_CLK     __HAL_RCC_GPIOA_CLK_ENABLE()
#define USER_PB_PIN                 GPIO_PIN_11
#define USER_PB_GPIO_PORT           GPIOA

#define GPI_EXTI1_IRQn              EXTI15_10_IRQn
#define GPI_EXTI1_IRQHandler        EXTI15_10_IRQHandler

#define GPI_EXTI2_IRQn              EXTI3_IRQn
#define GPI_EXTI2_IRQHandler        EXTI3_IRQHandler

#define GPI_EXTI3_IRQn              EXTI4_IRQn
#define GPI_EXTI3_IRQHandler        EXTI4_IRQHandler

#define GPO_TIMER                   TIM2
#define ENABLE_GPO_TIMER_CLK        __HAL_RCC_TIM2_CLK_ENABLE()
#define GPO_TIMER_IRQn              TIM2_IRQn
#define GPO_TIMER_IRQHandler        TIM2_IRQHandler

// private variables
TIM_HandleTypeDef    GPO_TIM_Handle;

// Pushbuttons callbacks
void (*reset_pb_callback)(void);
void (*user_pb_callback)(void);
void (*start_pb_callback)(void);
void (*mode_pb_callback)(void);
/* led period variables */
uint16_t led_tx_period;
uint16_t led_rx_period;
uint16_t led_usr1_period;
uint16_t led_usr2_period;
/* led blinking status */
uint8_t led_tx_blinking;
uint8_t led_rx_blinking;
uint8_t led_usr1_blinking;
uint8_t led_usr2_blinking;

// IRQ handler functions
void GPI_EXTI1_IRQHandler(void)
{
  if(__HAL_GPIO_EXTI_GET_IT(RESET_PB_PIN) != RESET)
  {
    __HAL_GPIO_EXTI_CLEAR_IT(RESET_PB_PIN);
    if(ram_data[BIOLOID_RESET_PB_CNTRL]&GPIO_INT_USED)
    {
      if(reset_pb_callback!=0)
        reset_pb_callback();
    }
    else
    {
      ram_data[BIOLOID_RESET_PB_CNTRL]|=GPIO_INT_FLAG;
      if(ram_data[BIOLOID_RESET_PB_CNTRL]&GPIO_INT_EN)
        bioloid_dyn_slave_set_int();
    }
  }
  if(__HAL_GPIO_EXTI_GET_IT(USER_PB_PIN) != RESET)
  {
    __HAL_GPIO_EXTI_CLEAR_IT(USER_PB_PIN);
    if(ram_data[BIOLOID_USER_PB_CNTRL]&GPIO_INT_USED)
    {
      if(user_pb_callback!=0)
        user_pb_callback();
    }
    else
    {
      ram_data[BIOLOID_USER_PB_CNTRL]|=GPIO_INT_FLAG;
      if(ram_data[BIOLOID_USER_PB_CNTRL]&GPIO_INT_EN)
        bioloid_dyn_slave_set_int();
    }
  }
}

void GPI_EXTI2_IRQHandler(void)
{
  if(__HAL_GPIO_EXTI_GET_IT(START_PB_PIN) != RESET)
  {
    __HAL_GPIO_EXTI_CLEAR_IT(START_PB_PIN);
    if(ram_data[BIOLOID_START_PB_CNTRL]&GPIO_INT_USED)
    {
      if(start_pb_callback!=0)
        start_pb_callback();
    }
    else
    {
      ram_data[BIOLOID_START_PB_CNTRL]|=GPIO_INT_FLAG;
      if(ram_data[BIOLOID_START_PB_CNTRL]&GPIO_INT_EN)
        bioloid_dyn_slave_set_int();
    }
  }
}

void GPI_EXTI3_IRQHandler(void)
{
  if(__HAL_GPIO_EXTI_GET_IT(MODE_PB_PIN) != RESET)
  {
    __HAL_GPIO_EXTI_CLEAR_IT(MODE_PB_PIN);
    if(ram_data[BIOLOID_MODE_PB_CNTRL]&GPIO_INT_USED)
    {
      if(mode_pb_callback!=0)
        mode_pb_callback();
    }
    else
    {
      ram_data[BIOLOID_MODE_PB_CNTRL]|=GPIO_INT_FLAG;
      if(ram_data[BIOLOID_MODE_PB_CNTRL]&GPIO_INT_EN)
        bioloid_dyn_slave_set_int();
    }
  }
}

void GPO_TIMER_IRQHandler(void)
{
  uint32_t capture;

  if(__HAL_TIM_GET_FLAG(&GPO_TIM_Handle, TIM_FLAG_CC1) != RESET)
  {
    if(__HAL_TIM_GET_IT_SOURCE(&GPO_TIM_Handle, TIM_IT_CC1) !=RESET)
    {
      __HAL_TIM_CLEAR_IT(&GPO_TIM_Handle, TIM_IT_CC1);
      capture = HAL_TIM_ReadCapturedValue(&GPO_TIM_Handle, TIM_CHANNEL_1);
      __HAL_TIM_SET_COMPARE(&GPO_TIM_Handle, TIM_CHANNEL_1, (capture + led_tx_period));
      HAL_GPIO_TogglePin(LED_TX_GPIO_PORT,LED_TX_PIN);
      if(ram_data[BIOLOID_TXD_LED_CNTRL]&GPIO_VALUE)
        ram_data[BIOLOID_TXD_LED_CNTRL]&=(~GPIO_VALUE);
      else
        ram_data[BIOLOID_TXD_LED_CNTRL]|=GPIO_VALUE;
    }
  }
  if(__HAL_TIM_GET_FLAG(&GPO_TIM_Handle, TIM_FLAG_CC2) != RESET)
  {
    if(__HAL_TIM_GET_IT_SOURCE(&GPO_TIM_Handle, TIM_IT_CC2) !=RESET)
    {
      __HAL_TIM_CLEAR_IT(&GPO_TIM_Handle, TIM_IT_CC2);
      capture = HAL_TIM_ReadCapturedValue(&GPO_TIM_Handle, TIM_CHANNEL_2);
      __HAL_TIM_SET_COMPARE(&GPO_TIM_Handle, TIM_CHANNEL_2, (capture + led_rx_period));
      HAL_GPIO_TogglePin(LED_RX_GPIO_PORT,LED_RX_PIN);
      if(ram_data[BIOLOID_RXD_LED_CNTRL]&GPIO_VALUE)
        ram_data[BIOLOID_RXD_LED_CNTRL]&=(~GPIO_VALUE);
      else
        ram_data[BIOLOID_RXD_LED_CNTRL]|=GPIO_VALUE;
    }
  }
  if(__HAL_TIM_GET_FLAG(&GPO_TIM_Handle, TIM_FLAG_CC3) != RESET)
  {
    if(__HAL_TIM_GET_IT_SOURCE(&GPO_TIM_Handle, TIM_IT_CC3) !=RESET)
    {
      __HAL_TIM_CLEAR_IT(&GPO_TIM_Handle, TIM_IT_CC3);
      capture = HAL_TIM_ReadCapturedValue(&GPO_TIM_Handle, TIM_CHANNEL_3);
      __HAL_TIM_SET_COMPARE(&GPO_TIM_Handle, TIM_CHANNEL_3, (capture + led_usr1_period));
      HAL_GPIO_TogglePin(LED_USR1_GPIO_PORT,LED_USR1_PIN);
      if(ram_data[BIOLOID_USER1_LED_CNTRL]&GPIO_VALUE)
        ram_data[BIOLOID_USER1_LED_CNTRL]&=(~GPIO_VALUE);
      else
        ram_data[BIOLOID_USER1_LED_CNTRL]|=GPIO_VALUE;
    }
  }
  if(__HAL_TIM_GET_FLAG(&GPO_TIM_Handle, TIM_FLAG_CC4) != RESET)
  {
    if(__HAL_TIM_GET_IT_SOURCE(&GPO_TIM_Handle, TIM_IT_CC4) !=RESET)
    {
      __HAL_TIM_CLEAR_IT(&GPO_TIM_Handle, TIM_IT_CC4);
      capture = HAL_TIM_ReadCapturedValue(&GPO_TIM_Handle, TIM_CHANNEL_4);
      __HAL_TIM_SET_COMPARE(&GPO_TIM_Handle, TIM_CHANNEL_4, (capture + led_usr2_period));
      HAL_GPIO_TogglePin(LED_USR2_GPIO_PORT,LED_USR2_PIN);
      if(ram_data[BIOLOID_USER2_LED_CNTRL]&GPIO_VALUE)
        ram_data[BIOLOID_USER2_LED_CNTRL]&=(~GPIO_VALUE);
      else
        ram_data[BIOLOID_USER2_LED_CNTRL]|=GPIO_VALUE;
    }
  }
  /* TIM Update event */
  if(__HAL_TIM_GET_FLAG(&GPO_TIM_Handle, TIM_FLAG_UPDATE) != RESET)
    if(__HAL_TIM_GET_IT_SOURCE(&GPO_TIM_Handle, TIM_IT_UPDATE) !=RESET)
      __HAL_TIM_CLEAR_IT(&GPO_TIM_Handle, TIM_IT_UPDATE);
  /* TIM Break input event */
  if(__HAL_TIM_GET_FLAG(&GPO_TIM_Handle, TIM_FLAG_BREAK) != RESET)
    if(__HAL_TIM_GET_IT_SOURCE(&GPO_TIM_Handle, TIM_IT_BREAK) !=RESET)
      __HAL_TIM_CLEAR_IT(&GPO_TIM_Handle, TIM_IT_BREAK);
  /* TIM Trigger detection event */
  if(__HAL_TIM_GET_FLAG(&GPO_TIM_Handle, TIM_FLAG_TRIGGER) != RESET)
    if(__HAL_TIM_GET_IT_SOURCE(&GPO_TIM_Handle, TIM_IT_TRIGGER) !=RESET)
      __HAL_TIM_CLEAR_IT(&GPO_TIM_Handle, TIM_IT_TRIGGER);
  /* TIM commutation event */
  if(__HAL_TIM_GET_FLAG(&GPO_TIM_Handle, TIM_FLAG_COM) != RESET)
    if(__HAL_TIM_GET_IT_SOURCE(&GPO_TIM_Handle, TIM_IT_COM) !=RESET)
      __HAL_TIM_CLEAR_IT(&GPO_TIM_Handle, TIM_FLAG_COM);
}

// private functions

// public functions
void gpio_init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  TIM_ClockConfigTypeDef sClockSourceConfig;
  TIM_MasterConfigTypeDef sMasterConfig;

  /* initialize the callback function pointers */
  reset_pb_callback=0;
  user_pb_callback=0;
  start_pb_callback=0;
  mode_pb_callback=0;
  led_tx_period=0;
  led_rx_period=0;
  led_usr1_period=0;
  led_usr2_period=0;
  led_tx_blinking=0;
  led_rx_blinking=0;
  led_usr1_blinking=0;
  led_usr2_blinking=0;

  /* enable clocks */
  ENABLE_LED_TX_GPIO_CLK;
  ENABLE_LED_RX_GPIO_CLK;
  ENABLE_LED_USR1_GPIO_CLK;
  ENABLE_LED_USR2_GPIO_CLK;
  ENABLE_START_PB_GPIO_CLK;
  ENABLE_MODE_PB_GPIO_CLK;
  ENABLE_RESET_PB_GPIO_CLK;
  ENABLE_USER_PB_GPIO_CLK;

  /* initlaize GPIO pins */
  GPIO_InitStructure.Pin = LED_TX_PIN;
  GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStructure.Speed = GPIO_SPEED_HIGH;
  GPIO_InitStructure.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(LED_TX_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.Pin = LED_RX_PIN;
  HAL_GPIO_Init(LED_RX_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.Pin = LED_USR1_PIN;
  HAL_GPIO_Init(LED_USR1_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.Pin = LED_USR2_PIN;
  HAL_GPIO_Init(LED_USR2_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.Pin = START_PB_PIN;
  GPIO_InitStructure.Mode = GPIO_MODE_IT_FALLING;
  GPIO_InitStructure.Speed = GPIO_SPEED_HIGH;
  HAL_GPIO_Init(START_PB_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.Pin = MODE_PB_PIN;
  HAL_GPIO_Init(MODE_PB_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.Pin = RESET_PB_PIN;
  HAL_GPIO_Init(RESET_PB_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.Pin = USER_PB_PIN;
  HAL_GPIO_Init(USER_PB_GPIO_PORT, &GPIO_InitStructure);

  /* enable external interrupts */
  // clear any pending interrupts
  if(__HAL_GPIO_EXTI_GET_IT(START_PB_PIN) != RESET)
    __HAL_GPIO_EXTI_CLEAR_IT(START_PB_PIN);
  if(__HAL_GPIO_EXTI_GET_IT(MODE_PB_PIN) != RESET)
    __HAL_GPIO_EXTI_CLEAR_IT(MODE_PB_PIN);
  if(__HAL_GPIO_EXTI_GET_IT(RESET_PB_PIN) != RESET)
    __HAL_GPIO_EXTI_CLEAR_IT(RESET_PB_PIN);
  if(__HAL_GPIO_EXTI_GET_IT(USER_PB_PIN) != RESET)
    __HAL_GPIO_EXTI_CLEAR_IT(USER_PB_PIN);
  HAL_NVIC_SetPriority(GPI_EXTI1_IRQn, 3, 3);
  HAL_NVIC_EnableIRQ(GPI_EXTI1_IRQn);

  HAL_NVIC_SetPriority(GPI_EXTI2_IRQn, 3, 3);
  HAL_NVIC_EnableIRQ(GPI_EXTI2_IRQn);

  HAL_NVIC_SetPriority(GPI_EXTI3_IRQn, 3, 3);
  HAL_NVIC_EnableIRQ(GPI_EXTI3_IRQn);

  /* LED's timer configuration */
  ENABLE_GPO_TIMER_CLK;

  GPO_TIM_Handle.Instance=GPO_TIMER;
  GPO_TIM_Handle.Init.Period = 0xFFFFFFFF;
  GPO_TIM_Handle.Init.Prescaler = 42000;
  GPO_TIM_Handle.Init.ClockDivision = TIM_CLOCKDIVISION_DIV2;
  GPO_TIM_Handle.Init.CounterMode = TIM_COUNTERMODE_UP;
  GPO_TIM_Handle.Init.RepetitionCounter=0;
  HAL_TIM_Base_Init(&GPO_TIM_Handle);
  // initialize the timer interrupts
  HAL_NVIC_SetPriority(GPO_TIMER_IRQn, 3, 3);
  HAL_NVIC_EnableIRQ(GPO_TIMER_IRQn);
  /* use the internal clock */
  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
  HAL_TIM_ConfigClockSource(&GPO_TIM_Handle, &sClockSourceConfig);
  HAL_TIM_OC_Init(&GPO_TIM_Handle);
  /* disable master/slave mode */
  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  HAL_TIMEx_MasterConfigSynchronization(&GPO_TIM_Handle, &sMasterConfig);
}

void gpio_set_led(led_t led_id)
{
  switch(led_id)
  {
    case TXD_LED:
      HAL_GPIO_WritePin(LED_TX_GPIO_PORT,LED_TX_PIN,GPIO_PIN_SET);
      break;
    case RXD_LED:
      HAL_GPIO_WritePin(LED_RX_GPIO_PORT,LED_RX_PIN,GPIO_PIN_SET);
      break;
    case USER1_LED:
      HAL_GPIO_WritePin(LED_USR1_GPIO_PORT,LED_USR1_PIN,GPIO_PIN_SET);
      break;
    case USER2_LED:
      HAL_GPIO_WritePin(LED_USR2_GPIO_PORT,LED_USR2_PIN,GPIO_PIN_SET);
      break;
  }
}

void gpio_clear_led(led_t led_id)
{
  switch(led_id)
  {
    case TXD_LED:
      HAL_GPIO_WritePin(LED_TX_GPIO_PORT,LED_TX_PIN,GPIO_PIN_RESET);
      break;
    case RXD_LED:
      HAL_GPIO_WritePin(LED_RX_GPIO_PORT,LED_RX_PIN,GPIO_PIN_RESET);
      break;
    case USER1_LED:
      HAL_GPIO_WritePin(LED_USR1_GPIO_PORT,LED_USR1_PIN,GPIO_PIN_RESET);
      break;
    case USER2_LED:
      HAL_GPIO_WritePin(LED_USR2_GPIO_PORT,LED_USR2_PIN,GPIO_PIN_RESET);
      break;
  }
}

void gpio_toggle_led(led_t led_id)
{
  switch(led_id)
  {
    case TXD_LED:
      HAL_GPIO_TogglePin(LED_TX_GPIO_PORT,LED_TX_PIN);
      break;
    case RXD_LED:
      HAL_GPIO_TogglePin(LED_RX_GPIO_PORT,LED_RX_PIN);
      break;
    case USER1_LED:
      HAL_GPIO_TogglePin(LED_USR1_GPIO_PORT,LED_USR1_PIN);
      break;
    case USER2_LED:
      HAL_GPIO_TogglePin(LED_USR2_GPIO_PORT,LED_USR2_PIN);
      break;
  }
}

void gpio_blink_led(led_t led_id, int16_t period_ms)
{
  TIM_OC_InitTypeDef TIM_OCInitStructure;
  uint32_t capture;

  TIM_OCInitStructure.OCMode = TIM_OCMODE_TIMING;
  TIM_OCInitStructure.OCPolarity = TIM_OCPOLARITY_HIGH;
  TIM_OCInitStructure.OCFastMode = TIM_OCFAST_DISABLE;

  switch(led_id)
  {
    case TXD_LED:
      if(period_ms>=1)
      {
        led_tx_period=period_ms;
        if(!led_tx_blinking)
        {
          capture = HAL_TIM_ReadCapturedValue(&GPO_TIM_Handle, TIM_CHANNEL_1);
          TIM_OCInitStructure.Pulse = capture+led_tx_period;
          HAL_TIM_OC_ConfigChannel(&GPO_TIM_Handle, &TIM_OCInitStructure,TIM_CHANNEL_1);
          HAL_TIM_OC_Start_IT(&GPO_TIM_Handle, TIM_CHANNEL_1);
          led_tx_blinking=1;
        }
      }
      else
      {
        HAL_TIM_OC_Stop_IT(&GPO_TIM_Handle, TIM_CHANNEL_1);
        led_tx_blinking=0;
      }
      break;
    case RXD_LED:
      if(period_ms>=1)
      {
        led_rx_period=period_ms;
        if(!led_rx_blinking)
        {
          capture = HAL_TIM_ReadCapturedValue(&GPO_TIM_Handle, TIM_CHANNEL_2);
          TIM_OCInitStructure.Pulse = capture+led_rx_period;
          HAL_TIM_OC_ConfigChannel(&GPO_TIM_Handle, &TIM_OCInitStructure,TIM_CHANNEL_2);
          HAL_TIM_OC_Start_IT(&GPO_TIM_Handle, TIM_CHANNEL_2);
          led_tx_blinking=1;
        }
      }
      else
      {
        HAL_TIM_OC_Stop_IT(&GPO_TIM_Handle, TIM_CHANNEL_2);
        led_tx_blinking=0;
      }
      break;
    case USER1_LED:
      if(period_ms>=1)
      {
        led_usr1_period=period_ms;
        if(!led_usr1_blinking)
        {
          capture = HAL_TIM_ReadCapturedValue(&GPO_TIM_Handle, TIM_CHANNEL_3);
          TIM_OCInitStructure.Pulse = capture+led_usr1_period;
          HAL_TIM_OC_ConfigChannel(&GPO_TIM_Handle, &TIM_OCInitStructure,TIM_CHANNEL_3);
          HAL_TIM_OC_Start_IT(&GPO_TIM_Handle, TIM_CHANNEL_3);
          led_usr1_blinking=1;
        }
      }
      else
      {
        HAL_TIM_OC_Stop_IT(&GPO_TIM_Handle, TIM_CHANNEL_3);
        led_usr1_blinking=0;
      }
      break;
    case USER2_LED:
      if(period_ms>=1)
      {
        led_usr2_period=period_ms;
        if(!led_usr2_blinking)
        {
          capture = HAL_TIM_ReadCapturedValue(&GPO_TIM_Handle, TIM_CHANNEL_4);
          TIM_OCInitStructure.Pulse = capture+led_usr2_period;
          HAL_TIM_OC_ConfigChannel(&GPO_TIM_Handle, &TIM_OCInitStructure,TIM_CHANNEL_4);
          HAL_TIM_OC_Start_IT(&GPO_TIM_Handle, TIM_CHANNEL_4);
          led_usr2_blinking=1;
        }
      }
      else
      {
        HAL_TIM_OC_Stop_IT(&GPO_TIM_Handle, TIM_CHANNEL_4);
        led_usr2_blinking=0;
      }
      break;
  }
}

uint8_t gpio_is_pushbutton_pressed(pushbutton_t pb_id)
{
  switch(pb_id)
  {
    case START_PB:
      if(HAL_GPIO_ReadPin(START_PB_GPIO_PORT,START_PB_PIN)==GPIO_PIN_SET)
        return 0x01;
      else
        return 0x00;
      break;
    case MODE_PB:
      if(HAL_GPIO_ReadPin(MODE_PB_GPIO_PORT,MODE_PB_PIN)==GPIO_PIN_SET)
        return 0x01;
      else
        return 0x00;
      break;
    case RESET_PB:
      if(HAL_GPIO_ReadPin(RESET_PB_GPIO_PORT,RESET_PB_PIN)==GPIO_PIN_SET)
        return 0x01;
      else
        return 0x00;
      break;
    case USER_PB:
      if(HAL_GPIO_ReadPin(USER_PB_GPIO_PORT,USER_PB_PIN)==GPIO_PIN_SET)
        return 0x01;
      else
        return 0x00;
      break;
  }

  return 0x00;
}

void gpio_set_pushbutton_callback(pushbutton_t pb_id,void (*callback)(void))
{
  switch(pb_id)
  {
    case START_PB:
      start_pb_callback=callback;
      break;
    case MODE_PB:
      mode_pb_callback=callback;
      break;
    case RESET_PB:
      reset_pb_callback=callback;
      break;
    case USER_PB:
      user_pb_callback=callback;
      break;
  }
}

// operation functions
uint8_t gpio_in_range(unsigned short int address,unsigned short int length)
{
  return ram_in_window(GPIO_BASE_ADDRESS,GPIO_MEM_LENGTH,address,length);
}

void gpio_process_read_cmd(unsigned short int address,unsigned short int length,unsigned char *data)
{
  if(ram_in_range(BIOLOID_RESET_PB_CNTRL,address,length))
    if(!(ram_data[BIOLOID_RESET_PB_CNTRL]&GPIO_INT_USED))/* GPIO is not internally used */
    {
      if(ram_data[BIOLOID_RESET_PB_CNTRL]&GPIO_INT_FLAG)
      {
        ram_data[BIOLOID_RESET_PB_CNTRL]&=(~GPIO_INT_FLAG);
        if(ram_data[BIOLOID_RESET_PB_CNTRL]&GPIO_INT_EN)
          bioloid_dyn_slave_clear_int();
      }
    } 
  if(ram_in_range(BIOLOID_USER_PB_CNTRL,address,length))
    if(!(ram_data[BIOLOID_USER_PB_CNTRL]&GPIO_INT_USED))/* GPIO is not internally used */
    {
      if(ram_data[BIOLOID_USER_PB_CNTRL]&GPIO_INT_FLAG)
      {
        ram_data[BIOLOID_USER_PB_CNTRL]&=(~GPIO_INT_FLAG);
        if(ram_data[BIOLOID_USER_PB_CNTRL]&GPIO_INT_EN)
          bioloid_dyn_slave_clear_int();
      }
    } 
  if(ram_in_range(BIOLOID_START_PB_CNTRL,address,length))
    if(!(ram_data[BIOLOID_START_PB_CNTRL]&GPIO_INT_USED))/* GPIO is not internally used */
    {
      if(ram_data[BIOLOID_START_PB_CNTRL]&GPIO_INT_FLAG)
      {
        ram_data[BIOLOID_START_PB_CNTRL]&=(~GPIO_INT_FLAG);
        if(ram_data[BIOLOID_START_PB_CNTRL]&GPIO_INT_EN)
          bioloid_dyn_slave_clear_int();
      }
    } 
  if(ram_in_range(BIOLOID_MODE_PB_CNTRL,address,length))
    if(!(ram_data[BIOLOID_MODE_PB_CNTRL]&GPIO_INT_USED))/* GPIO is not internally used */
    {
      if(ram_data[BIOLOID_MODE_PB_CNTRL]&GPIO_INT_FLAG)
      {
        ram_data[BIOLOID_MODE_PB_CNTRL]&=(~GPIO_INT_FLAG);
        if(ram_data[BIOLOID_MODE_PB_CNTRL]&GPIO_INT_EN)
          bioloid_dyn_slave_clear_int();
      }
    } 
}

void gpio_process_write_cmd(unsigned short int address,unsigned short int length,unsigned char *data)
{
  uint16_t period;

  /* LED's */
  if(ram_in_range(BIOLOID_USER1_LED_CNTRL,address,length))
  {
    if(!(ram_data[BIOLOID_USER1_LED_CNTRL]&GPIO_INT_USED))/* GPIO is not internally used */
    {
      period=led_usr1_period;
      if(ram_in_range(BIOLOID_USER1_LED_PERIOD_L,address,length))
        period=(period&0xFF00)+data[BIOLOID_USER1_LED_PERIOD_L-address];
      if(ram_in_range(BIOLOID_USER1_LED_PERIOD_H,address,length))
        period=(period&0x00FF)+(data[BIOLOID_USER1_LED_PERIOD_H-address]<<8);
      if(data[BIOLOID_USER1_LED_CNTRL-address]&GPIO_BLINK)
        gpio_blink_led(USER1_LED,period);
      else
      {
        gpio_blink_led(USER1_LED,0x0000);
        if(data[BIOLOID_USER1_LED_CNTRL-address]&GPIO_TOGGLE)
        {
          gpio_toggle_led(USER1_LED);
          if(ram_data[BIOLOID_USER1_LED_CNTRL]&GPIO_VALUE)
            ram_data[BIOLOID_USER1_LED_CNTRL]&=(~GPIO_VALUE);
          else
            ram_data[BIOLOID_USER1_LED_CNTRL]|=GPIO_VALUE;
        }
        else
        {
          if(data[BIOLOID_USER1_LED_CNTRL-address]&GPIO_VALUE)
          {
            gpio_set_led(USER1_LED);
            ram_data[BIOLOID_USER1_LED_CNTRL]|=GPIO_VALUE;
          }
          else
          {
            gpio_clear_led(USER1_LED);
            ram_data[BIOLOID_USER1_LED_CNTRL]&=(~GPIO_VALUE);
          }
        }
      }
    }
  }  
  if(ram_in_range(BIOLOID_USER2_LED_CNTRL,address,length))
  {
    if(!(ram_data[BIOLOID_USER2_LED_CNTRL]&GPIO_INT_USED))/* GPIO is not internally used */
    {
      period=led_usr2_period;
      if(ram_in_range(BIOLOID_USER2_LED_PERIOD_L,address,length))
        period=(period&0xFF00)+data[BIOLOID_USER2_LED_PERIOD_L-address];
      if(ram_in_range(BIOLOID_USER2_LED_PERIOD_H,address,length))
        period=(period&0x00FF)+(data[BIOLOID_USER2_LED_PERIOD_H-address]<<8);
      if(data[BIOLOID_USER2_LED_CNTRL-address]&GPIO_BLINK)
        gpio_blink_led(USER2_LED,period);
      else
      {
        gpio_blink_led(USER2_LED,0x0000);
        if(data[BIOLOID_USER2_LED_CNTRL-address]&GPIO_TOGGLE)
        {
          gpio_toggle_led(USER2_LED);
          if(ram_data[BIOLOID_USER2_LED_CNTRL]&GPIO_VALUE)
            ram_data[BIOLOID_USER2_LED_CNTRL]&=(~GPIO_VALUE);
          else
            ram_data[BIOLOID_USER2_LED_CNTRL]|=GPIO_VALUE;
        }
        else
        {
          if(data[BIOLOID_USER2_LED_CNTRL-address]&GPIO_VALUE)
          {
            gpio_set_led(USER2_LED);
            ram_data[BIOLOID_USER2_LED_CNTRL]|=GPIO_VALUE;
          }
          else
          {
            gpio_clear_led(USER2_LED);
            ram_data[BIOLOID_USER2_LED_CNTRL]&=(~GPIO_VALUE);
          }
        }
      }
    }
  }  
  if(ram_in_range(BIOLOID_RXD_LED_CNTRL,address,length))
  {
    if(!(ram_data[BIOLOID_RXD_LED_CNTRL]&GPIO_INT_USED))/* GPIO is not internally used */
    {
      period=led_rx_period;
      if(ram_in_range(BIOLOID_RXD_LED_PERIOD_L,address,length))
        period=(period&0xFF00)+data[BIOLOID_RXD_LED_PERIOD_L-address];
      if(ram_in_range(BIOLOID_RXD_LED_PERIOD_H,address,length))
        period=(period&0x00FF)+(data[BIOLOID_RXD_LED_PERIOD_H-address]<<8);
      if(data[BIOLOID_RXD_LED_CNTRL-address]&GPIO_BLINK)
        gpio_blink_led(RXD_LED,period);
      else
      {
        gpio_blink_led(RXD_LED,0x0000);
        if(data[BIOLOID_RXD_LED_CNTRL-address]&GPIO_TOGGLE)
        {
          gpio_toggle_led(RXD_LED);
          if(ram_data[BIOLOID_RXD_LED_CNTRL]&GPIO_VALUE)
            ram_data[BIOLOID_RXD_LED_CNTRL]&=(~GPIO_VALUE);
          else
            ram_data[BIOLOID_RXD_LED_CNTRL]|=GPIO_VALUE;
        }
        else
        {
          if(data[BIOLOID_RXD_LED_CNTRL-address]&GPIO_VALUE)
          {
            gpio_set_led(RXD_LED);
            ram_data[BIOLOID_RXD_LED_CNTRL]|=GPIO_VALUE;
          }
          else
          {
            gpio_clear_led(RXD_LED);
            ram_data[BIOLOID_RXD_LED_CNTRL]&=(~GPIO_VALUE);
          }
        }
      }
    }
  }  
  if(ram_in_range(BIOLOID_TXD_LED_CNTRL,address,length))
  {
    if(!(ram_data[BIOLOID_TXD_LED_CNTRL]&GPIO_INT_USED))/* GPIO is not internally used */
    {
      period=led_tx_period;
      if(ram_in_range(BIOLOID_TXD_LED_PERIOD_L,address,length))
        period=(period&0xFF00)+data[BIOLOID_TXD_LED_PERIOD_L-address];
      if(ram_in_range(BIOLOID_TXD_LED_PERIOD_H,address,length))
        period=(period&0x00FF)+(data[BIOLOID_TXD_LED_PERIOD_H-address]<<8);
      if(data[BIOLOID_TXD_LED_CNTRL-address]&GPIO_BLINK)
        gpio_blink_led(TXD_LED,period);
      else
      {
        gpio_blink_led(TXD_LED,0x0000);
        if(data[BIOLOID_TXD_LED_CNTRL-address]&GPIO_TOGGLE)
        {
          gpio_toggle_led(TXD_LED);
          if(ram_data[BIOLOID_TXD_LED_CNTRL]&GPIO_VALUE)
            ram_data[BIOLOID_TXD_LED_CNTRL]&=(~GPIO_VALUE);
          else
            ram_data[BIOLOID_TXD_LED_CNTRL]|=GPIO_VALUE;
        }
        else
        {
          if(data[BIOLOID_TXD_LED_CNTRL-address]&GPIO_VALUE)
          {
            gpio_set_led(TXD_LED);
            ram_data[BIOLOID_TXD_LED_CNTRL]|=GPIO_VALUE;
          }
          else
          {
            gpio_clear_led(TXD_LED);
            ram_data[BIOLOID_TXD_LED_CNTRL]&=(~GPIO_VALUE);
          }
        }
      }
    }
  }
  /* pushbuttons */  
  if(ram_in_range(BIOLOID_RESET_PB_CNTRL,address,length))
    if(!(ram_data[BIOLOID_RESET_PB_CNTRL]&GPIO_INT_USED))/* GPIO is not internally used */
      ram_data[BIOLOID_RESET_PB_CNTRL]=data[BIOLOID_RESET_PB_CNTRL-address];
  if(ram_in_range(BIOLOID_USER_PB_CNTRL,address,length))
    if(!(ram_data[BIOLOID_USER_PB_CNTRL]&GPIO_INT_USED))/* GPIO is not internally used */
      ram_data[BIOLOID_USER_PB_CNTRL]=data[BIOLOID_USER_PB_CNTRL-address];
  if(ram_in_range(BIOLOID_START_PB_CNTRL,address,length))
    if(!(ram_data[BIOLOID_START_PB_CNTRL]&GPIO_INT_USED))/* GPIO is not internally used */
      ram_data[BIOLOID_START_PB_CNTRL]=data[BIOLOID_START_PB_CNTRL-address];
  if(ram_in_range(BIOLOID_MODE_PB_CNTRL,address,length))
    if(!(ram_data[BIOLOID_MODE_PB_CNTRL]&GPIO_INT_USED))/* GPIO is not internally used */
      ram_data[BIOLOID_MODE_PB_CNTRL]=data[BIOLOID_MODE_PB_CNTRL-address];
}
