#include "zigbee.h"
#include "ram.h"

#define ENABLE_ZIGBEE_PWR_GPIO_CLK      __HAL_RCC_GPIOC_CLK_ENABLE()
#define ZIGBEE_PWR_PIN                  GPIO_PIN_12
#define ZIGBEE_PWR_GPIO_PORT            GPIOC

// private variables

// public functions
void zigbee_init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  ENABLE_ZIGBEE_PWR_GPIO_CLK;  

  /* initlaize GPIO pins */
  GPIO_InitStructure.Pin = ZIGBEE_PWR_PIN;
  GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStructure.Speed = GPIO_SPEED_HIGH;
  GPIO_InitStructure.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(ZIGBEE_PWR_GPIO_PORT, &GPIO_InitStructure);
  // disable power by default
  zigbee_disable_power();
}

void zigbee_enable_power(void)
{
  HAL_GPIO_WritePin(ZIGBEE_PWR_GPIO_PORT,ZIGBEE_PWR_PIN,GPIO_PIN_RESET);
  ram_data[BIOLOID_ZIGBEE_CNTRL]|=ZIGBEE_EN_PWR;
}

void zigbee_disable_power(void)
{
  HAL_GPIO_WritePin(ZIGBEE_PWR_GPIO_PORT,ZIGBEE_PWR_PIN,GPIO_PIN_SET);
  ram_data[BIOLOID_ZIGBEE_CNTRL]&=(~ZIGBEE_EN_PWR);
}

void zigbee_enable(void)
{
  ram_data[BIOLOID_ZIGBEE_CNTRL]|=ZIGBEE_ENABLE;
}

void zigbee_disable(void)
{
  ram_data[BIOLOID_ZIGBEE_CNTRL]&=(~ZIGBEE_ENABLE);
}

// operation functions
uint8_t zigbee_in_range(unsigned short int address,unsigned short int length)
{
  return ram_in_window(ZIGBEE_BASE_ADDRESS,ZIGBEE_MEM_LENGTH,address,length);  
}

void zigbee_process_write_cmd(unsigned short int address,unsigned short int length,unsigned char *data)
{
  if(ram_in_range(BIOLOID_ZIGBEE_CNTRL,address,length))
  {
    if(data[BIOLOID_ZIGBEE_CNTRL-address]&ZIGBEE_EN_PWR)
      zigbee_enable_power();
    else
      zigbee_disable_power();
    if(data[BIOLOID_ZIGBEE_CNTRL-address]&ZIGBEE_ENABLE)
      zigbee_enable();
    else
      zigbee_disable();
  }
}

